from ftw.protectinactive.registry import IProtectInactiveSettings
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def migrate_to_1001(portal):
    registry = getUtility(IRegistry)
    registry.registerInterface(IProtectInactiveSettings)



from AccessControl.unauthorized import Unauthorized
from datetime import datetime
from ftw.protectinactive.registry import IProtectInactiveSettings
from plone.app.dexterity.behaviors.metadata import IPublication
from plone import api
from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.interfaces import ISiteRoot
from zExceptions import NotFound
from zope.component.hooks import getSite


def protect_incative_hook(event):
    """ Protect inactive content from unauthorized access. """

    site = getSite()
    if not site:
        return

    if not is_hook_active():
        return

    context = findContext(event.request)

    for permission in ['Modify portal content', 'Request review', 'Review portal content']:
        if api.user.has_permission(permission, obj=context):
            return

    publication_date, expiration_date = getPublicationDates(context)

    if isUnreleased(publication_date) or isExpired(expiration_date):
        # raise configured exception
        exception_type = api.portal.get_registry_record(
            name='exception_type', interface=IProtectInactiveSettings)
        exception = {
            'Unauthorized': Unauthorized,
            'NotFound': NotFound
        }.get(exception_type, NotImplementedError)

        raise exception()


def is_hook_active():
    try:
        is_active = api.portal.get_registry_record(
            name='is_active',
            interface=IProtectInactiveSettings,
            default=True
        )
    except KeyError:
        return True
    return is_active


def isUnreleased(publication_date):
    """ Checks if the publication date is in the future.
        If it is it checks if the user has access to unreleased content.
    """
    now = datetime.now()
    return (publication_date and
            now < publication_date and
            not api.user.has_permission('Access future portal content'))


def isExpired(expiration_date):
    """ Checks if the expiration date has been exceeded.
        If it is it check if the user has access to expired content.
    """
    now = datetime.now()
    return (expiration_date and
            now > expiration_date and
            not api.user.has_permission('Access inactive portal content'))


def getPublicationDates(context):
    """ Returns the publication and the expiration dates.
    """
    publication = IPublication(context, None)
    if not publication:  # IPublication is not supported
        return None, None

    return publication.effective, publication.expires


def findContext(request):
    """Find the context from the request
       copied from: https://github.com/plone/plone.app.theming/blob/master/src/plone/app/theming/utils.py#L171
    """
    published = request.get('PUBLISHED', None)
    context = getattr(published, '__parent__', None)
    if context is not None:
        return context

    for parent in request.PARENTS:
        if IContentish.providedBy(parent) or ISiteRoot.providedBy(parent):
            return parent

    return request.PARENTS[0]



from AccessControl import Unauthorized
from datetime import datetime
from datetime import timedelta
from ftw.builder import Builder
from ftw.builder import create
from ftw.protectinactive.registry import IProtectInactiveSettings
from ftw.protectinactive.tests import FunctionalTestCase
from ftw.protectinactive.tests.dx_folder import setup_dx_folder
from ftw.testbrowser import browsing
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import transaction


class TestTraversalHook(FunctionalTestCase):

    def setUp(self):
        super(TestTraversalHook, self).setUp()
        self.grant('Manager')

        setup_dx_folder(self.portal)

        # no dates
        self.no_dates_dxfolder = create(Builder('dx folder'))

        # active
        dxeffective = datetime.now() - timedelta(days=10)
        dxexpiration = datetime.now() + timedelta(days=10)

        self.active_dxfolder = create(Builder('dx folder')
                                      .having(effective=dxeffective,
                                              expires=dxexpiration))

        dxeffective = datetime.now() - timedelta(days=20)
        dxexpiration = datetime.now() - timedelta(days=10)

        self.past_dxfolder = create(Builder('dx folder')
                                    .having(effective=dxeffective,
                                            expires=dxexpiration))

        # future content
        dxeffective = datetime.now() + timedelta(days=10)
        dxexpiration = datetime.now() + timedelta(days=20)

        self.future_dxfolder = create(Builder('dx folder')
                                      .having(effective=dxeffective,
                                              expires=dxexpiration))

    @browsing
    def test_manager_can_access_everything(self, browser):
        browser.login()
        try:
            browser.open(self.no_dates_dxfolder)
            browser.open(self.active_dxfolder)
            browser.open(self.past_dxfolder)
            browser.open(self.future_dxfolder)
        except Unauthorized:
            self.fail("A Manager has to be able to access all content.")

    @browsing
    def test_editor_can_access_everything(self, browser):
        editor = create(Builder('user').with_roles('Editor', on=self.portal))
        browser.login(editor.getId())

        try:
            browser.open(self.no_dates_dxfolder)
            browser.open(self.active_dxfolder)
            browser.open(self.past_dxfolder)
            browser.open(self.future_dxfolder)
        except Unauthorized:
            self.fail("An Editor has to be able to access the content.")

    @browsing
    def test_normal_user_access_rights(self, browser):
        user = create(Builder('user'))
        browser.login(user.getId())

        try:
            browser.open(self.no_dates_dxfolder)
            browser.open(self.active_dxfolder)
        except Unauthorized:
            self.fail("A normal user has to be able to access active content.")

        with browser.expect_unauthorized():
            browser.open(self.past_dxfolder)
        with browser.expect_unauthorized():
            browser.open(self.future_dxfolder)

    @browsing
    def test_user_with_inactive_rights_can_access_past_content(self, browser):
        user = create(Builder('user'))
        browser.login(user.getId())
        self.portal.manage_permission('Access inactive portal content', user.getRoles())
        transaction.commit()

        try:
            browser.open(self.past_dxfolder)
        except Unauthorized:
            self.fail("User has inactive rights but can't access inactive content.")

        with browser.expect_unauthorized():
            browser.open(self.future_dxfolder)

    @browsing
    def test_user_with_future_rights_can_access_future_content(self, browser):
        user = create(Builder('user'))
        browser.login(user.getId())
        self.portal.manage_permission('Access future portal content', user.getRoles())
        transaction.commit()

        try:
            browser.open(self.future_dxfolder)
        except Unauthorized:
            self.fail("User has future rights but can't access future content.")

        with browser.expect_unauthorized():
            browser.open(self.past_dxfolder)

    @browsing
    def test_user_can_access_active_content_within_an_inactive_container(self, browser):
        self.nested_dxfolder = create(Builder('dx folder')
                                      .within(self.past_dxfolder))

        user = create(Builder('user'))
        browser.login(user.getId())

        try:
            browser.open(self.nested_dxfolder)
        except Unauthorized:
            self.fail("User has to be able to access active content within "
                      "an inactive container.")

    @browsing
    def test_disable_plugin(self, browser):
        with browser.expect_unauthorized():
            browser.open(self.past_dxfolder)
        with browser.expect_unauthorized():
            browser.open(self.future_dxfolder)

        self._disable_hook()

        try:
            browser.open(self.past_dxfolder)
            browser.open(self.future_dxfolder)
        except Unauthorized:
            self.fail("A normal user can access the content, since the plugin is disabled")

    def _disable_hook(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IProtectInactiveSettings, check=False)
        settings.is_active = False
        transaction.commit()



from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope import schema


_ = MessageFactory('plone')


class IProtectInactiveSettings(Interface):
    exception_type = schema.Choice(
        title=_(u"exception_type", default=u"Exception Type"),
        description=_(
            u"exception_type_description",
            default=u"The exception raised when a user tries to access "
                    u"inactive content without permission."),
        values=['Unauthorized', 'NotFound'],
        default='Unauthorized'
    )

    is_active = schema.Bool(
        title=_(u"is_active", default=u"Globally enable/disable hook"),
        default=True,
        required=False,
        missing_value=True,
    )



from ftw.builder.dexterity import DexterityBuilder
from ftw.builder import registry
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.dexterity.fti import DexterityFTI
from plone.supermodel import model
from zope.interface.declarations import alsoProvides
from zope.interface import implements
from zope.interface import Interface


class IDexterityContainerSchema(model.Schema):
    pass

alsoProvides(IDexterityContainerSchema, IFormFieldProvider)


class IDexterityFolder(Interface):
    pass


class DexterityFolder(Container):
    implements(IDexterityFolder)


class DexterityFolderBuilder(DexterityBuilder):
    portal_type = 'DexterityFolder'


registry.builder_registry.register('dx folder', DexterityFolderBuilder)


def setup_dx_folder(portal):

    types_tool = portal.portal_types

    # Simplelayout Container
    fti = DexterityFTI('DexterityFolder')
    fti.schema = 'ftw.protectinactive.tests.dx_folder.IDexterityContainerSchema'
    fti.klass = 'ftw.protectinactive.tests.dx_folder.DexterityFolder'
    fti.behaviors = (
        'plone.app.dexterity.behaviors.metadata.IBasic',
        'plone.app.content.interfaces.INameFromTitle',
        'plone.app.dexterity.behaviors.metadata.IPublication')
    types_tool._setObject('DexterityFolder', fti)

    site_fti = types_tool.get('Plone Site')
    site_fti.allowed_content_types = ('DexterityFolder')
